"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.reviveTape = exports.persistTape = exports.redactRequestHeaders = exports.Persistence = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const js_yaml_1 = __importDefault(require("js-yaml"));
const path_1 = __importDefault(require("path"));
const compression_1 = require("./compression");
const http_1 = require("./http");
/**
 * Persistence layer to save tapes to disk and read them from disk.
 */
class Persistence {
    constructor(tapeDir, redactHeaders) {
        this.tapeDir = tapeDir;
        this.redactHeaders = redactHeaders;
    }
    /**
     * Saves the tape to disk.
     */
    saveTapeToDisk(tapeName, tapeRecords) {
        const persistedTapeRecords = tapeRecords
            .map(this.redact, this)
            .map(persistTape);
        const tapePath = this.getTapePath(tapeName);
        fs_extra_1.default.ensureDirSync(path_1.default.dirname(tapePath));
        fs_extra_1.default.writeFileSync(tapePath, js_yaml_1.default.dump({
            http_interactions: persistedTapeRecords,
        }), "utf8");
    }
    /**
     * Redacts the request headers of the given record, depending on the redactHeaders array
     */
    redact(record) {
        redactRequestHeaders(record, this.redactHeaders);
        return record;
    }
    /**
     * Loads the tape from disk.
     */
    loadTapeFromDisk(tapeName) {
        const tapePath = this.getTapePath(tapeName);
        if (!fs_extra_1.default.existsSync(tapePath)) {
            throw new Error(`No tape found with name ${tapeName}`);
        }
        const persistedTapeRecords = js_yaml_1.default.load(fs_extra_1.default.readFileSync(tapePath, "utf8")).http_interactions;
        return persistedTapeRecords.map(reviveTape);
    }
    isTapeNameValid(tapeName) {
        return !path_1.default
            .relative(this.tapeDir, path_1.default.join(this.tapeDir, tapeName))
            .startsWith("../");
    }
    /**
     * Returns the tape's path on disk.
     */
    getTapePath(tapeName) {
        return path_1.default.join(this.tapeDir, `${tapeName}.yml`);
    }
}
exports.Persistence = Persistence;
/**
 * Redacts the headers of the given record based on the provided array of headers to redact
 */
function redactRequestHeaders(record, redactHeaders) {
    redactHeaders.forEach((header) => {
        if (record.request.headers[header]) {
            record.request.headers[header] = "XXXX";
        }
    });
}
exports.redactRequestHeaders = redactRequestHeaders;
function persistTape(record) {
    return {
        request: {
            method: record.request.method,
            path: record.request.path,
            headers: record.request.headers,
            body: serialiseForTape(record.request),
        },
        response: {
            status: record.response.status,
            headers: record.response.headers,
            body: serialiseForTape(record.response),
        },
    };
}
exports.persistTape = persistTape;
function reviveTape(persistedRecord) {
    return {
        request: {
            method: persistedRecord.request.method,
            path: persistedRecord.request.path,
            headers: persistedRecord.request.headers,
            body: unserialiseBuffer(persistedRecord.request.body),
        },
        response: {
            status: persistedRecord.response.status,
            headers: persistedRecord.response.headers,
            body: unserialiseBuffer(persistedRecord.response.body),
        },
    };
}
exports.reviveTape = reviveTape;
function serialiseForTape(r) {
    const buffer = (0, http_1.getHttpBodyDecoded)(r);
    const contentEncoding = (0, http_1.getHttpContentEncoding)(r);
    const compressionAlgorithm = (0, compression_1.convertHttpContentEncodingToCompressionAlgorithm)(contentEncoding);
    const utf8Representation = buffer.toString("utf8");
    try {
        // Can it be safely stored and recreated in YAML?
        const recreatedBuffer = Buffer.from(js_yaml_1.default.load(js_yaml_1.default.dump(utf8Representation)), "utf8");
        if (Buffer.compare(buffer, recreatedBuffer) === 0) {
            // Yes, we can store it in YAML.
            return {
                encoding: "utf8",
                data: utf8Representation,
                compression: compressionAlgorithm,
            };
        }
    }
    catch (_a) {
        // Fall through.
    }
    // No luck. Fall back to Base64, persisting the original buffer
    // since we might as well store it in its compressed state.
    return {
        encoding: "base64",
        data: r.body.toString("base64"),
    };
}
function unserialiseBuffer(persisted) {
    let buffer;
    switch (persisted.encoding) {
        case "base64":
            buffer = Buffer.from(persisted.data, "base64");
            break;
        case "utf8":
            buffer = Buffer.from(persisted.data, "utf8");
            buffer = (0, compression_1.compressBuffer)(persisted.compression || "none", buffer);
            break;
        default:
            throw new Error(`Unsupported encoding!`);
    }
    return buffer;
}
